package geniusweb.simplerunner;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotNull;

import java.io.IOException;
import java.net.URISyntaxException;

import org.junit.Before;
import org.junit.Test;

import geniusweb.actions.Action;
import geniusweb.connection.ConnectionEnd;
import geniusweb.inform.Inform;
import geniusweb.references.PartyRef;

public class ClassPathConnectionFactoryTest {
	private ClassPathConnectionFactory factory;
	private PartyRef PARTYREF;

	@Before
	public void before() throws URISyntaxException {
		factory = new ClassPathConnectionFactory();
		PARTYREF = new PartyRef(
				"classpath:geniusweb.exampleparties.randomparty.RandomParty");

	}

	@Test(expected = IllegalArgumentException.class)
	public void testWrongURI() throws IOException, URISyntaxException {
		factory.connect(new PartyRef("http://blabla"));
	}

	@Test(expected = IllegalArgumentException.class)
	public void testNullPath() throws IOException, URISyntaxException {
		// bad because classpath should not start with //
		factory.connect(new PartyRef("http://some.class.path"));
	}

	@Test(expected = IllegalArgumentException.class)
	public void testUnknownParty() throws IOException, URISyntaxException {
		factory.connect(new PartyRef("classpath:blabla.bla"));
	}

	@Test
	public void testRandomParty() throws IOException, URISyntaxException {
		ConnectionEnd<Action, Inform> party = factory.connect(PARTYREF);
		assertNotNull(party);
	}

	@Test
	public void testConnectionPairProtocolClosesNormally()
			throws InterruptedException {
		ConnectionPair cpair = new ConnectionPair(PARTYREF);
		assertEquals(2, cpair.getOpenConnections().size());
		cpair.getProtocolToPartyConn().close();
		Thread.sleep(100);

		assertEquals(0, cpair.getOpenConnections().size());

	}

	@Test
	public void testConnectionPairProtocolClosesWithExc()
			throws InterruptedException {
		ConnectionPair cpair = new ConnectionPair(PARTYREF);
		assertEquals(2, cpair.getOpenConnections().size());
		cpair.getProtocolToPartyConn().setError(new Error("test"));
		Thread.sleep(100);
		assertEquals(0, cpair.getOpenConnections().size());
	}

	@Test
	public void testConnectionPairPartyClosesNormally()
			throws InterruptedException {
		ConnectionPair cpair = new ConnectionPair(PARTYREF);
		assertEquals(2, cpair.getOpenConnections().size());
		cpair.getPartyToProtocolConn().close();
		Thread.sleep(100);

		assertEquals(0, cpair.getOpenConnections().size());

	}

	@Test
	public void testConnectionPairPartyClosesWithExc()
			throws InterruptedException {
		ConnectionPair cpair = new ConnectionPair(PARTYREF);
		assertEquals(2, cpair.getOpenConnections().size());
		cpair.getPartyToProtocolConn().setError(new Error("test"));
		Thread.sleep(100);
		assertEquals(0, cpair.getOpenConnections().size());
	}

}
